/**
 * Created by jinglupeng on 10/23/15.
 */

// init pdf.js
global.window = global;
global.navigator = {userAgent: 'node'};
global.PDFJS = {};
global.DOMParser = require('./domparsermock.js').DOMParserMock;
require('./pdf.combined.js');
require('./domstubs.js');
PDFJS.workerSrc = true;

var fs = require('fs');


function parserPage(page) {
    return new Promise(function (resolve) {
        var json = {};
        var viewport = page.getViewport(1);
        json.width = viewport.width;
        json.height = viewport.height;
        json.rotation = viewport.rotation;

        page.getTextContent().then(function (textContent) {
            var styles = textContent.styles || [];
            var texts = [];

            var items = textContent.items || [];
            for (var i = 0; i < items.length; i++) {
                var item = items[i];

                // filter
                var content = item.str;
                if (!/\S/.test(content)) {
                    continue;
                }

                var text = {};

                var tx = PDFJS.Util.transform(
                    PDFJS.Util.transform(viewport.transform, item.transform),
                    [1, 0, 0, -1, 0, 0]);
                var style = styles[item.fontName];

                // angle
                var angle = Math.atan2(tx[1], tx[0]);
                if (style.vertical) {
                    angle += Math.PI / 2;
                }
                // font height
                var fontHeight = Math.sqrt((tx[2] * tx[2]) + (tx[3] * tx[3]));
                var fontAscent = fontHeight;
                if (style.ascent) {
                    fontAscent = style.ascent * fontAscent;
                } else if (style.descent) {
                    fontAscent = (1 + style.descent) * fontAscent;
                }

                // left and top
                var left;
                var top;
                if (angle === 0) {
                    left = tx[4];
                    top = tx[5] - fontAscent;
                } else {
                    left = tx[4] + (fontAscent * Math.sin(angle));
                    top = tx[5] - (fontAscent * Math.cos(angle));
                }

                text.left = left;
                text.top = top;
                text.fontSize = fontHeight;
                text.fontFamily = style.fontFamily;
                text.content = item.str;
                text.width = item.width;
                text.height = item.height;
                texts.push(text);
            }
            json.texts = texts;
            resolve(json);
        });
    });
}

module.exports = function (path) {
    return new Promise(function (resolve, reject) {
        var data = new Uint8Array(fs.readFileSync(path));
        PDFJS.getDocument(data).then(function (doc) {
            var nums = doc.pdfInfo.numPages;
            var docJSON = {
                nums: nums,
                page: []
            };
            for (var i = 0; i < nums; i++) {
                (function (num) {
                    doc.getPage(num).then(function (page) {
                        parserPage(page).then(function (json) {
                            docJSON.page[num - 1] = json || {};
                            for (var j = 0; j < nums; j++) {
                                if (docJSON.page[j] === null || docJSON.page[j] === undefined) {
                                    break;
                                }
                            }
                            if (j === nums) {
                                // Finish
                                resolve(docJSON);
                            }
                        })
                    });
                })(i + 1);
            }
        }).then(function () {
        }, function (err) {
            reject(err);
        });
    });
}
